<?php
/**
 * Pagination
 * 
 * @package Pagination's class 
 * @author Gabriel MATIAS <gabr.matias [at gmail] somethingagainstspam [and tld:] com>
 * @copyright Gab.Matias
 * @version 1.0
 * @access public
 */

class Pagination{
    
    var $url;
    var $url_var;
    var $current_page;
    var $total_items;
    var $total_pages;
    var $limit;
    var $id, $i_id;
    var $sort;
    
    /**
     * Pagination::Pagination()
     * 
     * @param string $id
     * @param integer $total_items
     * @param integer $limit
     * @param string $url_var
     * @param integer $sort (1= ASC, 0 = DESC)
     * @param mixed $page_request
     * @return
     */
    function Pagination($id = 'default', $total_items = 0, $limit = 0, $url_var = 'current_page', $sort = 1, $page_request = NULL){
        if(!isset($_SESSION['paginate']['instances_'.$id])){
            $_SESSION['paginate']['instances_'.$id]=0;
        }
        $_SESSION['paginate']['instances_'.$id]++;
        $this->i_id = $id.$_SESSION['paginate']['instances_'.$id];
        $this->id=$id;
        
        if($url_var==$id)
            $this->url_var='page_'.$this->i_id;
        else
            $this->url_var=$url_var;
        
        // Allow to use POST or whatever to define current pagenumber
        $this->request     = !empty($page_request[$this->url_var]) ? $page_request : $_GET;
        $this->sort        = $sort;
        
        $this->limit       = ((int) $limit > 0) ? (int) $limit : 0;
        $this->total_items = ((int) $total_items > 0) ? (int) $total_items : 0;
        $this->total_pages = ($this->limit)?(floor($this->total_items/$this->limit)+(boolean)($this->total_items%$this->limit)):0;
        if(!empty($this->request[$this->url_var]))
            $_SESSION['paginate'][$this->i_id] = $this->setCurrent_Page($this->request[$this->url_var]);
        else
            $_SESSION['paginate'][$this->i_id] = $this->setCurrent_Page((isset($_SESSION['paginate'][$this->i_id])) ? $_SESSION['paginate'][$this->i_id] : '');
    }
    
    /**
     * Pagination::setCurrent_Page()
     * 
     * @param mixed $page_number
     * @return
     */
    function setCurrent_Page($page_number){
            if(ctype_digit($page_number) && $page_number > 0 && $page_number <= $this->total_pages)
                $this->current_page = $page_number;
            else{
                if(!$this->sort)
                    $this->current_page = $this->total_pages;
                else
                    $this->current_page = 1;
            }
        return $this->current_page;
    }
    
    /**
     * Pagination::setUrl()
     * 
     * @param mixed $url
     * @return
     */
    public function setUrl($url){
        $this->url=$url;
    }
    
    /**
     * Pagination::assignArray()
     * 
     * @param mixed $smarty
     * @param string $varname
     * @return
     */
    public function assignArray(&$smarty, $varname='paginate'){
        if(is_object($smarty) && (strtolower(get_class($smarty)) == 'smarty' || is_subclass_of($smarty, 'smarty'))) {
            $array['page_current'] = $this->current_page;
            $array['page_total'] = $this->total_pages;
            $array['total'] = $this->total_items;
            $array['first'] = ($this->limit*($this->current_page-1)+1);
            $array['last'] = ($this->current_page>=$this->total_pages)?$this->total_items:($this->current_page*$this->limit);
            $array['size'] = $array['last'] - $array['first'];
            $array['url'] = $this->url;
            $array['urlvar'] = $this->url_var;
            $array['current_item'] = $array['first'];
            $array['limit'] = $this->limit;
            
            $smarty->assign($varname, $array);
        } else {
            trigger_error("Pagination: [assign] A valid Smarty object is needed.");
            return false;            
        }
        
        return $array;
    } 
    
    static function validArray($array){
        if(is_array($array) && isset($array['page_current'], $array['page_total'], $array['first'], $array['last'], $array['size'],
        $array['url'], $array['urlvar'], $array['current_item'], $array['limit'])){
            return true;
        }
        return false;
    }
    
    public function getLimit(){
        return $this->limit;
    }
    public function getIndex(){
        return $this->limit*($this->current_page-1);
    }
    /**
     * Pagination::__destruct()
     * 
     * @return
     */
    function __destruct(){
        if (isset($_SESSION['paginate']['instances_'.$this->id]))
            if($this->id.$_SESSION['paginate']['instances_'.$this->id]==$this->i_id)
                unset($_SESSION['paginate']['instances_'.$this->id]);
    }
}
?>