<?php

class GuestbookDbHandler
{
	
	/**
	 * @var object pear_ADOConnection
	 */
	var $db;
	/**
	 * @var module table prefix
	 */
	var $table_prefix;
	/**
	 * @var string table holding the guestbook entries
	 */
	var $entry_table;
	/**
	 * @var string table holding the guestbooks
	 */
	var $guestbook_table;
	
	/**
	 * Class constructor 
	 * @param object pear_ADOConnection $db
	 */
	function GuestbookDbHandler(&$db)
	{
		$this->setDb($db);
		$this->setTablePrefix(cms_db_prefix() . 'module_guestbook_');
		$this->setEntryTable(cms_db_prefix() . 'module_guestbook');
		$this->setGuestbookTable($this->getTablePrefix() . 'guestbook');
	}
	
	/**
	 * @return object pear_ADOConnection
	 */
	function GetDb()
	{
		$db =& $this->db;
		return $db;
	}
	
	/**
	 * @param $db
	 * @return NULL
	 */
	function SetDb(&$db)
	{
		$this->db =& $db;
	}
	
	/**
	 * @return string
	 */
	function GetTablePrefix()
	{
		return $this->table_prefix;
	}
	
	/**
	 * @param string $prefix
	 * @return NULL
	 */
	function SetTablePrefix($prefix)
	{
		$this->table_prefix = $prefix;
	}
	
	/**
	 * @return string
	 */
	function GetEntryTable()
	{
		return $this->entry_table;
	}
	
	/**
	 * @param string $table
	 * @return NULL
	 */
	function SetEntryTable($table)
	{
		$this->entry_table = $table;
	}
	
	/**
	 * @return string
	 */
	function GetGuestbookTable()
	{
		return $this->guestbook_table;
	}
	
	/**
	 * @param string $table
	 * @return NULL
	 */
	function SetGuestbookTable($table)
	{
		$this->guestbook_table = $table;
	}
	
	function GetMessageGuestbooks()
	{
		$table = $this->entry_table;
		$query = "
			SELECT DISTINCT guestbook 
			FROM $table 
			ORDER BY guestbook
		";
		$result = $this->db->Execute($query);
		return $result;		
	}
	
	/**
	 * @param mixed $guestbook_id NULL for all guestbooks otherwise a guestbook ID
	 * @param boolean $count_inactive Whether inactive messages should be counted 
	 * @return object pear_ResultSet
	 */
	function GetNumberOfEntries($guestbook_id=NULL, $count_inactive=false)
	{
		$values = array();
		$where_clause = array();
		$query = "
			SELECT 
				COUNT(1) AS count
			FROM {$this->entry_table}"
		;
		if ($count_inactive == false)
		{
			$where_clause[] = "approved = '1'";
		}
		if (isset($guestbook_id))
		{
			$where_clause[] = "guestbook = ?";
			$values[] = $guestbook_id;
		}
		foreach($where_clause as $index => $where_part)
		{
			$query .= ($index == 0) ? " WHERE $where_part" :" AND $where_part";
		}

		$result = $this->db->Execute($query, $values);
		return $result;
	}
	
	/**
	 * @param mixed $guestbook_id
	 * @param string $order_by ('DESC' or 'ASC')
	 * @param boolean $get_inactive
	 * @param mixed $limit
	 * @param mixed $offset
	 * @return object pear_ResultSet
	 */
	function GetEntries($guestbook_id=NULL, $order_by='', $get_inactive=false, $limit=NULL, $offset=NULL)
	{
		$values = array();
		$where_clause = array();
		$query = '
			SELECT entry_id, sender, e_mail, homepage, country, location, message, date, approved, ip_address, guestbook, '.$this->guestbook_table.'.name as guestbook_name
			FROM `'.$this->entry_table.'` LEFT JOIN `'.$this->guestbook_table.'` ON `'.$this->entry_table.'`.`guestbook` = '.$this->guestbook_table.'.`id`
		';
		if ($get_inactive == false)
		{
			$where_clause[] = 'approved = 1';
		}
		if (isset($guestbook_id))
		{
			$where_clause[] = 'guestbook = ?';
			$values[] = $guestbook_id;
		}
		foreach($where_clause as $index => $where_part)
		{
			if ($index == 0)
			{
				$query .= ' WHERE '.$where_part;
			}
			else
			{
				$query .= ' AND '.$where_part;
			}
		}
		$query .= ($order_by != 'ASC' ? ' ORDER BY date DESC, entry_id DESC' : ' ORDER BY date ASC, entry_id ASC');

		if (isset($limit))
		{
			$query .= ' LIMIT '.$limit;
		}
		if (isset($offset))
		{
			$query .= ' OFFSET '.$offset;
		}
		
		$result = $this->db->Execute($query, $values);
		return $result;
	}
	
	/**
	 * @param $guestbook_id
	 * @param $entry
	 * @param $errors
	 * @return object pear_ResultSet
	 */
	function InsertEntry($guestbook_id, &$entry)
	{
		$guestbook_id = (int) $guestbook_id;
		
		$new_id = $this->db->GenID($this->GetTablePrefix() . 'seq');
		$entry->timestamp = $this->db->DBTimeStamp(($entry->timestamp == '' ? time() : $entry->timestamp));
		$query = '
			INSERT INTO '.$this->entry_table.'
			(
				entry_id, 
				guestbook, 
				sender, 
				e_mail, 
				homepage, 
				message, 
				date, 
				approved, 
				country, 
				location, 
				ip_address
			)
            	VALUES 
            (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, "'.$_SERVER['REMOTE_ADDR'].'")
		';
        $values = array(
        	$new_id, 
        	$guestbook_id, 
        	$entry->sender->name, 
        	$entry->sender->email, 
        	$entry->sender->homepage, 
        	$entry->message, 
        	substr($entry->timestamp, 1, strlen($entry->timestamp)-2),
        	($entry->IsActive() ? '1' : '0'), 
        	$entry->sender->country, 
        	$entry->sender->location
        );

        $result = $this->db->Execute($query, $values);
        if ($result && $this->db->Affected_Rows() == 1)
        {
        	$entry->entry_id = $new_id;
        	$entry->guestbook_id = $guestbook_id;
        }
		return $result;
	}

	/**
	 * This function creates a UniqID
	 *
	 * @return int The inserted UniqID
	 */
	function CreateUniqID()
	{
		$new_id = $this->db->GenID($this->getTablePrefix() . 'post_seq');
		$uid = uniqid('');
		$query = '
			INSERT INTO '.$this->table_prefix.'post
			(id, idpost) VALUES ('.$new_id.', "'.$uid.'")
		';
		$this->db->Execute($query);
		return $uid;
	}
	
	function DeleteUniqID($id)
	{
		$query = '
			DELETE FROM '.$this->table_prefix.'post
			WHERE idpost = ?
		';
		$result = $this->db->Execute($query, array($id));
		return $result;
	}
	
	function CheckUniqID($id)
	{
		$query = '
			SELECT * FROM '.$this->table_prefix.'post
			WHERE idpost = ?
		';
		$result = $this->db->Execute($query, array($id));
		return $result;
	}
	
	function GetEntryById($entry_id)
	{
		$query = 'SELECT entry_id, sender, e_mail, homepage, country, location, message, date, approved, ip_address, guestbook, '.$this->guestbook_table.'.name as guestbook_name
			FROM `'.$this->entry_table.'` LEFT JOIN `'.$this->guestbook_table.'` ON `'.$this->entry_table.'`.`guestbook` = '.$this->guestbook_table.'.`id` WHERE entry_id = '.$entry_id;
		$result = $this->db->Execute($query);
		return $result;
	}
	
}

?>