<?php

define('CMS_GB_ERROR_NOSENDER'     , 'CMS_GB_ERROR_NOSENDER');
define('CMS_GB_ERROR_NOMESSAGE'    , 'CMS_GB_ERROR_NOMESSAGE');
define('CMS_GB_ERROR_UNIQUE'       , 'CMS_GB_ERROR_UNIQUE');
define('CMS_GB_ERROR_TOOSOON'      , 'CMS_GB_ERROR_TOOSOON');
define('CMS_GB_ERROR_INVALID_EMAIL', 'CMS_GB_ERROR_INVALIDEMAIL');
define('CMS_GB_ERROR_TOOLONG'      , 'CMS_GB_ERROR_TOOLONG');
define('CMS_GB_ERROR_CAPTCHA'      , 'CMS_GB_ERROR_CAPTCHA');
define('CMS_GB_ERROR_BADWORDS'     , 'CMS_GB_ERROR_BADWORDS');

/**
 * Guestbook module frontend
 */
class GuestbookFrontend
{
	/**
	 * @var object Reference to the CMS ADOdb connection object
	*/
	var $dbhandler;
	
	/**
	 * @var object Reference to the global Smarty object
	*/
	var $smarty;
	
	/**
	 * @var string Current mode
	*/
	var $mode;
	
	/**
	 * @var array Valid modes Currently 'guestbook', 'entrylist', 'form' and 'entry'
	*/
	var $valid_modes;
	
	/**
	 * @var boolean show smileys (in the messages and on the submit form)
	*/
	var $show_smileys;
	
	/**
	 * @var boolean show a captcha on the submit form
	*/
	var $show_captcha;
	
	/**
	 * @var boolean show "Insert entry" button instead of the insert form
	*/
	var $show_insertbutton;
	
	/**
	 * @var boolean hides the "Insert entry" button after a successful post
	*/
	var $hide_insertbutton_after_post;
	
	/**
	 * @var object contain the Guestbook Module object
	*/
	var $mod;
	
	/**
	 * Guestbook Frontend class constructor
	 * @param int $id
	 * @param mixed $returnid
	 * @param array $params
	 * @param object $dbhandler
	 * @param object $smarty
	 */
	function GuestbookFrontend(&$dbhandler, &$smarty)
	{
		$this->dbhandler =& $dbhandler;
		$this->cms = cmsms();
		$this->smarty = $this->cms->GetSmarty();
		if(isset($this->cms->modules)){
			$this->mod = $this->cms->modules['Guestbook']['object'];
		}else{
			$modops = $this->cms->GetModuleOperations();
			$this->mod = $modops->get_module_instance('Guestbook');
		}
		
		// define valid modes
		$valid_modes = array(
			'guestbook',
			'form',
			'entrylist',
			'entry'
		);
		$this->SetValidModes($valid_modes);
		$this->SetMode('guestbook');

		$this->SetShowSmileys(false);
		$this->SetShowCaptcha(false);
		$this->SetShowInsertButton(true);
		$this->SetHideInsertButtonAfterPost(true);
	}
	
	/**
	 * Initializes (determins and sets) the Frontend mode
	 * @param $params
	 * @return unknown_type
	 */
	function InitMode($params)
	{
		$mode = 'guestbook';
	
		if (isset($params['mode']))
		{
			if (in_array($params['mode'], $this->GetValidModes()))
			{
				$mode = $params['mode'];
			}
		}
		

		// set mode to 'entry' if show_entry is used in the HTTP request
		if (isset($_REQUEST['show_entry']))
		{
			$mode = 'entry';
			$params['show_entry'] = (int) $_REQUEST['show_entry'];
		}
		
		// set mode to 'entry' if the parameter show_entry is set
		if ($params['show_entry'] != '')
		{
			$mode = 'entry';
		}
		
		// set the Frontend mode
		$this->SetMode($mode);
	}
	
	/**
	 * @return array
	 */
	function GetValidModes()
	{
		return $this->valid_modes;
	}
	/**
	 * @param array
	 */
	function SetValidModes($modes)
	{
		$this->valid_modes = $modes;
	}
	
	/**
	 * @return string
	 */
	function GetMode()
	{
		return $this->mode;
	}
	/**
	 * @param string
	 * @return boolean
	 */
	function SetMode($mode)
	{
		if (! in_array($mode, $this->GetValidModes()))
		{
			return FALSE;
		}
		else
		{
			$this->mode = $mode;
			return TRUE;
		}
	}
	
	/**
	 * @param boolean
	 */
	function SetShowInsertButton($show_insertbutton)
	{
		$this->show_insertbutton = $show_insertbutton;
	}
	/**
	 * @return boolean
	*/
	function GetShowInsertButton()
	{
		return $this->show_insertbutton;
	}
	
	/**
	 * @param boolean
	*/
	function SetHideInsertButtonAfterPost($hide_insertbutton_after_post)
	{
		$this->hide_insertbutton_after_post = $hide_insertbutton_after_post;
	}
	/**
	 * @return boolean
	*/
	function GetHideInsertButtonAfterPost()
	{
		return $this->hide_insertbutton_after_post;
	}
	/**
	 * @param boolean
	*/
	function SetShowSmileys($show_smileys)
	{
		$this->show_smileys = $show_smileys;
	}
	/**
	 * @return boolean
	*/
	function GetShowSmileys()
	{
		return $this->show_smileys;
	}
	
	/**
	 * @param boolean
	*/
	function SetShowCaptcha($show_captcha)
	{
		$this->show_captcha = $show_captcha;
	}
	/**
	 * @return boolean
	*/
	function GetShowCaptcha()
	{
		return $this->show_captcha;
	}
	
	/**
	 * @param array
	*/
	function InitCaptcha($captcha)
	{
		$show_captcha = FALSE;
		if ($captcha && $this->mod->ModuleIsInstalled('Captcha'))
		{
			$show_captcha = TRUE;
		}
		$this->SetShowCaptcha($show_captcha);
		
	}
	
	/**
	 * Process a submitted guestbook entry
	 */
	function ProcessPost($id, $params, $returnid)
	{
        $allowed = '<strong>,<em>,<u>,<font>,<span>';
        if($this->mod->ModuleIsInstalled('FCKeditorX') && $this->mod->GetPreference('wysiwyg') == '1' && $this->getShowSmileys() && $this->mod->ModuleIsInstalled('Smileys'))
            $allowed .= ',<img>'; // TODO: Enhancement needed (for now, all image can be used)
		if (isset($params['entry_submitted']) && !isset($params['cancel'])) 
		{
			// Get a Captcha module instance if needed
			$captcha = NULL;
			if ($this->GetShowCaptcha()) 
			{
				$captcha =& $this->mod->getModuleInstance('Captcha');
			}
			
			// validate the entry
			$errors = $this->ValidatePost($params, $captcha);
		
			if (!empty($errors))
			{
				// show errors
				$gb_error_messages = array(
					CMS_GB_ERROR_NOSENDER      => 'fabsend',
					CMS_GB_ERROR_NOMESSAGE     => 'fmessag',
					CMS_GB_ERROR_UNIQUE        => 'fui',
					CMS_GB_ERROR_TOOSOON       => 'toosoon',
					CMS_GB_ERROR_INVALID_EMAIL => 'femail',
					CMS_GB_ERROR_TOOLONG       => 'tolong',
					CMS_GB_ERROR_CAPTCHA       => 'captcha_failure_message',
					CMS_GB_ERROR_BADWORDS      => 'badwords_message'
				);
				foreach($errors as $index => $gb_error)
				{
					switch (gettype($gb_error))
					{
						case 'string': 
							$errors[$index] = $this->mod->Lang($gb_error_messages[$gb_error]);
							break;
						case 'array':
							$errors[$index] = $this->mod->Lang($gb_error_messages[$gb_error[0]], $gb_error[1]);
							break;
					}
				}
				$this->Errors = $errors;
				if ($errors[0] == $this->mod->Lang($gb_error_messages[CMS_GB_ERROR_UNIQUE]))
				{
					$this->SetShowInsertButton(true);
				}
				else
				{
					$this->SetShowInsertButton(false);
				}
			}
			else
			{
				// set the badwords parameter
				$params['badwords'] = explode(',', str_replace("\n",',', $this->mod->GetPreference('badwords')));
				foreach ($params['badwords'] as $index => $word)
				{
					if (trim($word) == '') {
						unset($params['badwords'][$index]);
					}
					else
					{
						$params['badwords'][$index] = trim($word);
					}
				}
				// apply bad word filter
				if (!empty($params['badwords']))
				{
					$alt_chars = array
						(
							'/a/i' => '[a@]',
							'/b/i' => '[b3]',
							'/i/i' => '[i1!|]',
							'/s/i' => '[s\$5]',
							'/t/i' => '[t+]',
							'/z/i' => '[z2]'
						);
					$badwords = $params['badwords'];
					array_walk($badwords, array($this, 'MakeRegex'), $alt_chars);
					$params['message'] = preg_replace($badwords, $this->mod->GetPreference('badwords_replacement'), $params['message']);
				}
				
				{
					// strip HTML tags in these params var
                    $prms=array('sender_name' => '', 'sender_country' => '', 'sender_location' => '', 'sender_email' => '', 'sender_homepage' => '', 'message' => '');
					foreach ($params as $index => $value)
					{
						if ($index == 'message' && $this->mod->GetPreference('wysiwyg') == '1' && $this->mod->ModuleIsInstalled('FCKeditorX'))
						{
                            $params['message'] = strip_tags($value,$allowed);
						}elseif(isset($prms[$index])){
                            $params[$index] = strip_tags($value);
						}
					}
				}
				
				// prepare the entry for insertion
				require_once(cms_join_path($this->mod->getModulePath(), 'lib', 'classes', 'module', 'class.GuestbookEntry.php'));
				require_once(cms_join_path($this->mod->getModulePath(), 'lib', 'classes', 'module', 'class.GuestbookEntrySender.php'));
				$sender = new GuestbookEntrySender(
					isset($params['sender_name'])?$params['sender_name']:'',
					isset($params['sender_email']) ? $params['sender_email'] : '',
					$_SERVER['REMOTE_ADDR'],
					isset($params['sender_country'])  ? $params['sender_country'] : '',
					isset($params['sender_location']) ? $params['sender_location'] : '',
					isset($params['sender_homepage']) ? $params['sender_homepage'] : ''
				);
				
				
				$entry = new GuestbookEntry(
					$sender,
					$params['message'],
					$params['shownow'] == '1' ? true : false
				);
				// insert the entry
				$this->mod->InsertEntry($params['guestbook'], $entry);
				
				// set the lastpost session variable
				$_SESSION['CMSGuestbook']['lastpost'] = time();
				
				// delete the unique ID from the database
				$this->dbhandler->DeleteUniqID($params['uid']);
				
				// assign a message to the frontend
				$this->Messages[] = $this->mod->Lang($entry->IsActive() ? 'showitnow' : 'showitlater');
				
				
				/**
				* E-Mailnotification
				*/
				if (isset($params['mailnoti']) && trim($params['mailnoti']) != '')
				{
				    $params['mailnoti'] = trim($params['mailnoti']);
                    $this->SendNotificationEmail($params);
				}
			}
		}
	}	
	
	function SendNotificationEmail($params)
	{
		global $gCms;
		if(strpos(str_replace("\n", ',', $params['mailnoti']), ',') !== FALSE){
		   $e_mailnoti_to = explode(',', $params['mailnoti']);
		}else{
		   $e_mailnoti_to = array($params['mailnoti']);
		}
        
		$e_mailnoti_subj = html_entity_decode($this->mod->Lang('e_mailnotibetr'), ENT_QUOTES, 'UTF-8');
		
		// create the notification E-mail message text
		$this->smarty->assign('notification_message', html_entity_decode($this->mod->Lang('e_mailnotimessage'),ENT_QUOTES,'UTF-8'));
	
		//  array: input field name => language field name; unfortunately email differs. 
		$fields = array(
			'sender_name' 	=> 'sender',
			'message' 	=> 'message',
			'sender_country' 	=> 'country',
			'sender_location' 	=> 'location',
			'sender_email'	=> 'email',
			'sender_homepage' 	=> 'homepage'); 
		
		foreach($fields as $field => $field_lang)
		{
			if ($params[$field] != '')
			{
				$titles[$field_lang] = html_entity_decode($this->mod->Lang($field_lang), ENT_QUOTES, 'UTF-8');
				$values[$field_lang] = $params[$field];
			}
		}

		$titles['module_link'] = html_entity_decode($this->mod->Lang('module_friendlyname'), ENT_QUOTES, 'UTF-8');
		$values['module_link'] = $this->mod->CreateReturnLink($this->smarty->id,get_pageid_or_alias_from_url(),'',array(),true);
		
		//CreateLink($id, 'default', get_pageid_or_alias_from_url(), '', array('guestbook' => $params['guestbook']), '', TRUE);
		
		$titles['admin_link'] = html_entity_decode($this->mod->Lang('title_mod_admin'), ENT_QUOTES, 'UTF-8');
		$values['admin_link'] = str_replace('&amp;','&',$this->mod->CreateLink('m1_', 'defaultadmin','', 'admininterface',array(), '',true));
		
		
		//$gCms->config['root_url'].'/'.$gCms->config['admin_dir'].'/moduleinterface.php?module=Guestbook';
		
		$this->smarty->assign('titles', $titles);
		$this->smarty->assign('values', $values);
		$msg = $this->mod->ProcessTemplateFromDatabase('guestbook-email');
		
		// send the mail
		# see if the CMSMailer module is installed
	
		$cmsmailer=cms_utils::get_module('CMSMailer');
		
		
		if ($cmsmailer)
		{
						
		
            foreach($e_mailnoti_to as $one_email){
                if($one_email != ''){
                    $cmsmailer->SetCharSet('UTF-8');
                    $cmsmailer->AddAddress($one_email);
                    $cmsmailer->SetSubject($e_mailnoti_subj);
        			$cmsmailer->SetBody($msg);
        			$cmsmailer->Send();
        			$cmsmailer->reset();
                }
            }
		} else {
			
			
			$headers = "MIME-Version: 1.0\nContent-Transfer-Encoding: 8bit\nContent-Type: text/plain; charset=\"UTF-8\"";
			$encoded_subject = "=?UTF-8?B?".base64_encode($e_mailnoti_subj)."?=\n";
			mail(
				str_replace("\n", ',', $params['mailnoti']), 	// address to send the mail to
				$encoded_subject, 	// E-mail subject
				$msg,			// The E-mail message
				$headers		// Additional mail headers
			);
		}
		
	}
	
	/**
	 * @param array $params
	 * @param object $captcha
	 * @return array Error messages if validation fails; empty array if validation successful
	 */
	function ValidatePost($params, &$captcha)
	{
		$errors = array();
		$timespan  = $params['timespan'];
		
		// Trim empty space
		$params['sender_name']  = (isset($params['sender_name'])  ? trim($params['sender_name'])  : '');
		$params['message']      = (isset($params['message'])      ? trim($params['message'])      : '');
		
		// Prevent page reload double post
		if (isset($params['uid']))
		{
			if(!$this->CheckUniqID($params['uid']))
			{
				$errors[] = CMS_GB_ERROR_UNIQUE;
				return $errors;
			}
		}
		
		// Check minimal timespan
		if (isset($_SESSION['CMSGuestbook']['lastpost']))
		{
			$time_between = time() - $_SESSION['CMSGuestbook']['lastpost'];
			if ($time_between < $timespan)
			{
				$errors[] = array(CMS_GB_ERROR_TOOSOON, array($time_between, $timespan));
			}
		}
		
		// Validate E-mail address
		if ($params['sender_email'] != '')
		{
			$expr = "/^([\w\!\#$\%\&'\*\+\-\/\=\?\^\`{\|\}\~]+\.)*[\w\!\#$\%\&'\*\+\-\/\=\?\^\`{\|\}\~]+@((((([a-z0-9]{1}[a-z0-9\-]{0,62}[a-z0-9]{1})|[a-z])\.)+[a-z]{2,6})|(\d{1,3}\.){3}\d{1,3}(\:\d{1,5})?)$/i"; // By James Watts and Francisco Jose Martin Moreno
			if (preg_match($expr, $params['sender_email']))
			{
				$emaildress = $params['sender_email'];
			}
			else
			{
				$errors[] = CMS_GB_ERROR_INVALID_EMAIL;
			}
		}
		// Check if fields are filled in
		if (!$params['sender_name'])
		{
			$errors[] = CMS_GB_ERROR_NOSENDER;
		}
		if (!$params['message'])
		{
			$errors[] = CMS_GB_ERROR_NOMESSAGE;
		}
		
		// Check if the message is not too long
		$maxlength = $params['textleng'];
		if ($maxlength > 0)
		{
			$lengofmessage = mb_strlen($params['message']);
			if ($lengofmessage > $maxlength)
			{
				$errors[] = CMS_GB_ERROR_TOOLONG;
			}
		}
		
		// Captha check
		if ($captcha != NULL) 
		{
			if (! $captcha->CheckCaptcha($params['captcha_phrase']))
			{
				$errors[] = CMS_GB_ERROR_CAPTCHA;
			}
		}
		return $errors;
	}
	
	/**
	 * Checks if an unique ID exists
	 *
	 * @param string $id The Uniqid to check for
	 * @return bool
	 */
	function CheckUniqID($id)
	{
		$result = $this->dbhandler->CheckUniqID($id);
		return ($result && $result->RecordCount() > 0) ? true : false;
	}
	
	function MakeRegex(&$word, $key, $altChars)
	{
		$word = ('/'.preg_replace(array_keys($altChars), $altChars, $word).'/i');
	}
	
	var $first=false;
	
	/**
	 * Outputs the guestbook frontend HTML
	 */
	function Show($id, $params, $returnid)
	{
		if(isset($params['asplugin']) && $params['asplugin']==1)
		{
			$this->first=true;
		}
		
		$this->SetShowSmileys($params['smiley'] == '1' ? true : false);
		$this->SetShowInsertButton($params['showaddform'] == 1  ? false : true);
		$this->SetHideInsertButtonAfterPost($params['hide_insertbutton_after_post'] == '1' ? true : false);
		$this->InitMode($params);
		switch($this->GetMode())
		{
			case 'form':
			case 'entrylist':
			case 'entry':
				$this->{'Show' . ucfirst($this->GetMode())}($id, $params, $returnid);
				break;
			default:
				
				$this->ShowGuestbook($id, $params, $returnid);
				
		}
		$this->first=true;
	}
	
	function ShowForm($id, $params, $returnid)
	{
		echo $this->GetFormHTML($id, $params, $returnid);
	}
	
	function GetFormHTML($id, $params, $returnid)
	{
		if ($this->GetShowInsertButton())
		{
			return $this->GetInsertButtonHTML($id, $params, $returnid);
		}
		else
		{
			return $this->GetEntryFormHTML($id, $params, $returnid);
		}
		return '';
	}
	
	function GetInsertButtonHTML($id, $params, $returnid)
	{
		$this->smarty->assign('formstart', $this->mod->CreateFormStart($id, 'default', $returnid, 'post', '', true, ''));
		$this->smarty->assign('hidden'   , $this->mod->CreateInputHidden($id, 'showaddform', '1'));
		$this->smarty->assign('submit'   , $this->mod->CreateInputSubmit($id, 'submit', $this->mod->Lang('neuein')));
		$this->smarty->assign('formend'  , $this->mod->CreateFormEnd());
		
		$this->smarty->assign('guestbook', $this);
		
		return $this->mod->ProcessTemplateFromDatabase('guestbook-showform-button');
	}
	
	function GetEntryFormHTML($id, $params, $returnid)
	{
		global $gCms;
		
		{ // assign titles
			$titles = array(
				'sender'          => $this->mod->Lang('inam'),
				'country'         => $this->mod->Lang('country'),
				'location'        => $this->mod->Lang('location'),
				'email'           => $this->mod->Lang('email'),
				'homepage'        => $this->mod->Lang('homepage'),
				'message'         => $this->mod->Lang('smessage'),
				'chars_remaining' => $this->mod->Lang('alength'),
                'id'              => $id
			);
			if ($this->GetShowSmileys())
			{
				$titles['smileys'] = $this->mod->Lang('title_table_smileys');
			}
			if ($this->GetShowCaptcha())
			{
				$titles['captcha'] = $this->mod->Lang('captcha_title');
			}
			$this->smarty->assign('titles', $titles);
		}

		{ // assign formstart
			$uid = (isset($params['uid']) ? $params['uid'] : $this->dbhandler->CreateUniqID());
			
			$this->smarty->assign('formstart', $this->mod->CreateFormStart($id, 'default', $returnid, 'post', '', true, '', array('showaddform'=>'1', 'uid'=>$uid)));
		}
		{ // assign the inputs
		
		$formarray=array('sender'=>'sender_name',
		'email'=>'sender_email',
		'homepage'=>'sender_homepage',
		'country'=>'sender_country',
		'location'=>'sender_location'
		
		);
			$inputs = array();
			foreach($formarray as $key=>$valueofparams)
			{
				$inputs[$key]   = $this->mod->CreateInputText($id, $valueofparams    , isset($params[$valueofparams])?$params[$valueofparams]:''    , '24', '255', '');
			}
			
			$inputs['message']  = $this->mod->CreateTextarea(FALSE, $id, isset($params['message'])?$params['message']:'', 'message', '', 'guestbook_message_input', '', '', '50', '5');
			$inputs['submit']   = $this->mod->CreateInputSubmit($id, 'entry_submitted', $this->mod->Lang('ginsert'));
    		if ($this->GetShowCaptcha())
    		{
                $inputs['captcha'] = $this->mod->CreateInputText($id, 'captcha_phrase', $params['captcha_phrase'], '24', '255', '');
    		}
			if ($this->Getmode() != 'form')
			{
				$inputs['cancel'] = $this->mod->CreateInputSubmit($id, 'cancel', $this->mod->Lang('cancel'));
			}
			
			$this->smarty->assign('inputs', $inputs);
		}

		// assign formend
		$this->smarty->assign('formend', $this->mod->CreateFormEnd());
		$formarray['message']='message';
		
		{ // assign values
			$values = array();
			foreach ($formarray as $field=>$fieldparams)
			{
				$values[$field] = isset($params['entry_submitted']) && isset($params[$field])&& !isset($params['cancel']) ? $params[$field] : '';
			}
			$this->smarty->assign('values'  , $values);
		}
		// counter
		$this->smarty->assign('counter', (!isset($params['cancel']) && isset($_POST['counter'])) ? $_POST['counter'] : $params['textleng']);
		$this->smarty->assign('max_textleng', $params['textleng']);
		
		$this->smarty->assign('toolong', $this->mod->Lang('tolong'));
		
		
		$labels['insert'] = $this->mod->Lang('ginsert');
		$labels['cancel'] = $this->mod->Lang('cancel');
		$this->smarty->assign('labels', $labels);
		$fckeditor='';
		if ($this->mod->ModuleIsInstalled('FCKeditorX') && $this->mod->GetPreference('wysiwyg') == '1')
		{
			require_once(cms_join_path($gCms->config['root_path'], 'modules', 'FCKeditorX', 'FCKeditor', 'fckeditor.php'));
			$oFCKeditor = new FCKeditor($id.'message');
            $this->smarty->assign('id', $id);
			$oFCKeditor->BasePath = $gCms->config['root_url'].'/modules/FCKeditorX/FCKeditor/';
			$oFCKeditor->Config['CustomConfigurationsPath'] = $gCms->config['root_url'].'/modules/Guestbook/fckconfig.js.php'.(($this->getShowSmileys() && $this->mod->ModuleIsInstalled('Smileys'))?'?smiley&amp;SmileyList=':'');
            $oFCKeditor->Config['EnterMode'] = 'br';
			$oFCKeditor->Value = $values['message'];
			$oFCKeditor->ToolbarSet = 'Guestbook';
            
	       /* Found this to get smiley working. I think it's not very clean but it work at all ;) */
			if ($this->getShowSmileys() && $this->mod->ModuleIsInstalled('Smileys'))
			{
				$smiley_handler =& $this->mod->GetSmileyHandler();
                $list =& $smiley_handler->GetSmileyList();
				$oFCKeditor->Config['SmileyPath'] = $gCms->config['root_url'] . $smiley_handler->GetBaseUrl();
                $txt='';
                $cnt=mb_strlen($oFCKeditor->Config['CustomConfigurationsPath']);
                foreach ($list as $smiley=>$url) 
				{
				    $temp  = str_replace('"', '&quot;', '"'.$smiley.'",');
                    if((mb_strlen($temp) + $cnt)<=4000)
				        $txt  .= $temp; else break;
				}
                $oFCKeditor->Config['CustomConfigurationsPath'] .= mb_substr($txt,0,-1);
			}
			else
			{
				
			}
			$fckeditor=$oFCKeditor->CreateHtml();
		}
		$this->smarty->assign('wysiwyg', $fckeditor);
		if ($this->GetShowCaptcha())
		{
			$captcha =& $this->mod->getModuleInstance('Captcha');
			$this->smarty->assign('captcha_title', $this->mod->Lang('captcha_title'));
			$this->smarty->assign('captcha', $captcha->getCaptcha());
		}
		
		if ($this->GetShowSmileys() && !($this->mod->ModuleIsInstalled('FCKeditorX') && $this->mod->GetPreference('wysiwyg') == '1'))
		{
			$html = "
			<script language=\"JavaScript\" type=\"text/javascript\">
            <!--
				function emoticon(text)
				{
					var textarea = document.getElementById('guestbook_message_input');
					// Attempt to create a text range (IE).
					if ((textarea.caretPos != null) && (textarea.createTextRange != null))
					{
						var caretPos = textarea.caretPos;
					
						caretPos.text = caretPos.text.charAt(caretPos.text.length - 1) == ' ' ? text + ' ' : text;
						caretPos.select();
					}
					// Firefox text range replace.
					else if (textarea.selectionStart != null)
					{
	                    text = ' ' + text + ' ';
						var begin = textarea.value.substr(0, textarea.selectionStart);
						var end = textarea.value.substr(textarea.selectionEnd);
						var scrollPos = textarea.scrollTop;
						
						textarea.value = begin + text + end;
						
						if (textarea.setSelectionRange)
						{
							textarea.focus();
							textarea.setSelectionRange(begin.length + text.length, begin.length + text.length);
						}
						textarea.scrollTop = scrollPos;
					}
					// Just put it on the end.
					else
					{
						textarea.value += ' ' + text;
						textarea.focus(textarea.value.length - 1);
					}
				}
            -->
			</script>
			";
			if ($this->mod->ModuleIsInstalled('Smileys'))
			{
				$smiley_handler =& $this->mod->GetSmileyHandler();
				$list = $smiley_handler->SmileysModuleInstance->GetSomeSmileys();
				if (count($list) > 0)
				{
					$html .= "<span class=\"cms-guestbook-somesmileys\">";
					foreach ($list as $smiley=>$url) 
					{
					    $smiley  = str_replace('"', "&quot;", $smiley); // escape " for proper HTML
					    $jsparam = str_replace("'", "\'"    , $smiley); // escape ' for proper JS call
					    $html .= "<a href=\"javascript:emoticon('$jsparam')\" title=\"$smiley\">$url</a>\n";
					}
					$html .= "</span>";
					$html .= $smiley_handler->SmileysModuleInstance->GetPopupExampleLink('','default','emoticon');
					$this->smarty->assign('smileys', $html);
				}
			} 
			else 
			{
				$this->smarty->assign('smiley_table_title', $this->mod->Lang('title_table_smileys'));
				$this->smarty->assign('smiley_dir'        , $gCms->config['root_url'] . '/modules/' . $this->mod->GetName() . '/images/smileys/default');
				$this->smarty->assign('smileys', $html . $this->mod->ProcessTemplate('smiley.tpl'));
			}
		}
		$this->smarty->assign('guestbook', $this);
		return $this->mod->ProcessTemplateFromDatabase('guestbook-form');
	}
	
	function ShowEntrylist($id, $params, $returnid)
	{
		echo $this->GetEntrylistHTML($id, $params, $returnid);
	}
	
	function GetEntrylistHTML($id, $params, $returnid)
	{
		// initialize session
		if (!isset($_SESSION))
		{
			session_start();
		}
	
		// initialize pagination
		{
            require_once(cms_join_path($this->mod->getModulePath(), 'lib', 'classes', 'Pagination.class.php'));
            $pag_id = $id . '_' . $returnid; // id used for pagination
            $limit = (isset($params['number']) && (int) $params['number'] > 0) ? (int) $params['number'] : 10;
            $pagination= new Pagination($pag_id, $this->mod->GetNumberOfEntries($params['guestbook']), $limit, $pag_id, ($params['sortasc'] == 'ASC' ? 0 : 1));
            $pagination->setUrl(($returnid!='__CMS_PREVIEW_PAGE__')?cmsms()->GetContentOperations()->LoadContentFromAlias($returnid)->GetURL():'http'.((!empty($_SERVER['HTTPS']))?'s' : '') . '://'.$_SERVER['SERVER_NAME'].preg_replace('#\?(.+)$#','',$_SERVER['REQUEST_URI']));
            $pagination->assignArray($this->smarty);
		}
			
		// get the entries to be shown
		$order_by = ($params['sortasc'] == '1' ? 'ASC' : 'DESC');
		$entries = $this->mod->GetEntries($pagination, (int) $params['guestbook'], $order_by);
		
		$this->smarty->assign('dateformat', isset($params['dateformat']) ? $params['dateformat'] : $this->mod->GetPreference('dateformat'));
		foreach ($entries as $entry_id => $entry)
		{
			if ($this->getShowSmileys())
			{
				$smiley_handler =& $this->mod->GetSmileyHandler();
				$entry->message = $smiley_handler->InsertSmileys($entry->message);
			}
			$entry->message = nl2br($entry->message);
			$entries[$entry_id] = $entry;
		}
		$this->smarty->assign('entries', $entries);
		
		// assign titles
		$titles = array(
			'date'      => $this->mod->Lang('sdate'),
			'time_hour' => $this->mod->Lang('uhr'),
			'sender'    => $this->mod->Lang('inam'),
			'country'   => $this->mod->Lang('country'),
			'location'  => $this->mod->Lang('location'),
			'e_mail'    => $this->mod->Lang('email'),
			'homepage'  => $this->mod->Lang('homepage'),
			'message'   => $this->mod->Lang('smessage')
		);
		$this->smarty->assign('titles', $titles);
		
		// assign pagination labels
		$this->smarty->assign('first_text', $this->mod->Lang('first_text'));
		$this->smarty->assign('prev_text', $this->mod->Lang('prev_text'));
		$this->smarty->assign('next_text', $this->mod->Lang('next_text'));
		$this->smarty->assign('last_text', $this->mod->Lang('last_text'));
		$this->smarty->assign('pagination_message', $this->mod->Lang('message'));
		$this->smarty->assign('pagination_of', $this->mod->Lang('pagination_of'));

		$this->smarty->assign('pag_id', $pag_id);

		// process the template
		return $this->mod->ProcessTemplateFromDatabase('guestbook-entrylist');
	}
	
	function ShowEntry($id, $params, $returnid)
	{
		$this->mod->DoAction('show-entry', $id, $params, $returnid);
	}

	function ShowGuestbook($id, $params, $returnid)
	{
		echo $this->GetGuestbookHTML($id, $params, $returnid);
	}
	
	function GetGuestbookHTML($id, $params, $returnid)
	{
		
		$this->smarty->assign('form', $this->GetFormHTML($id, $params, $returnid));
		$this->smarty->assign('entrylist', $this->GetEntrylistHTML($id, $params, $returnid));
		
		return $this->mod->ProcessTemplateFromDatabase('guestbook');
	}
	
}

?>
