<?php
#CMS - CMS Made Simple
#(c)2004 by Ted Kulp (wishy@users.sf.net)
#This project's homepage is: http://cmsmadesimple.sf.net
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#$Id$

/**
 * The Upgrade function
 *
 * If your module version number does not match the version<br/>
 * number of the installed module, the CMS Admin will give<br/>
 * you a chance to upgrade the module. This is the function<br/>
 * that actually handles the upgrade.<br/>
 * Ideally, this function should handle upgrades incrementally,<br/>
 * so you could upgrade from version 0.0.1 to 10.5.7 with<br/>
 * a single call. For a great example of this, see the News<br/>
 * module in the standard CMS install.<br/>
*/

// No direct access
if (!isset($gCms)) 
{
	exit();
}

// Get a reference to the CMS ADOConnection object
$db =& $this->cms->GetDb();

// Set the current_version variable
$current_version = $oldversion;

// Prevent the guestbook-form being set more than once
$form_template_is_installed = false;

// Upgrade the module
switch($current_version)
{
    case "1.0-rc1":
    {
        $dict = NewDataDictionary($db);
        
        # note: The last parameter I(INT) is not as documented but the only way I get ADOdb's RenameColumnSQL to return working sql 
        # (the parameter only influences MySQL)
        $sqlarray = $dict->RenameColumnSQL('`' . cms_db_prefix() . 'module_guestbook`', '`id`', '`entry_id`', 'I(INT)');
        $dict->ExecuteSQLArray($sqlarray);
	
        $sqlarray = $dict->AlterColumnSQL('`' . cms_db_prefix() . 'module_guestbook_post`', '`id` I');
        $dict->ExecuteSQLArray($sqlarray);

        $db->CreateSequence(cms_db_prefix() . "module_guestbook_post_seq");
        
        $this->UpdateStylesheet($db);

        $current_version = "1.0 RC2";
    }
    case '1.0 RC2':
    {
        $current_version = '1.0';
    }
    case '1.0':
    {
        $current_version = '1.0.1';
    }
    case '1.0.1':
    {
        $current_version = '1.0.2';
    }
    case '1.0.2':
    {
        $current_version = '1.0.3';
    }
    case '1.0.3':
    {
        $current_version = '1.0.4';
    }
    case '1.0.4':
    {
        $dict = NewDataDictionary($db);
        $flds = '
            location C(255) NOTNULL
        ';
        $taboptarray = array('mysql' => 'TYPE=MyISAM');
        $sqlarray = 
            $dict->AddColumnSQL(
                cms_db_prefix().'module_guestbook', 
                $flds, 
                $taboptarray
            );
        $dict->ExecuteSQLArray($sqlarray);

        $current_version = '1.1-beta1';
    }
    case '1.1-beta1':
    {
        $current_version = '1.1';
    }
    case '1.1':
    {
        $current_version = '1.1.1';
    }
    case '1.1.1':
    {
        $current_version = '1.1.2';
    }
	case '1.1.2':
		# set preferences
		$this->SetPreference('number', '10');
		$this->SetPreference('textleng', '1000');
		$this->SetPreference('timespan', '0');
		$this->SetPreference('sortasc', '0');
		$this->SetPreference('dateformat', 'd-m-Y');
		$this->SetPreference('shownow', '0');
		$this->SetPreference('smiley', '0');
		$this->SetPreference('searchsmileys', '0');
		$this->SetPreference('captcha', '0');
		$this->SetPreference('showaddform', '0');
		$this->SetPreference('badwords', '');
		$this->SetPreference('badwords_replacement', '####');
		
		# Setup the guestbook (frontend) templates
		include_once($this->GetModulePath().'/function.templates.php');
		foreach (array('guestbook', 'guestbook-entrylist', 'guestbook-form', 'guestbook-email', 'guestbook-showform-button') as $template)
		{
			$this->SetTemplate($template, GetGuestbookHtmlTemplate($template,$this->Lang('error_filenotfound_defaulttemplate')));
		}
		$form_template_is_installed = true;
		
		// update the stylesheet iserted on install
		# get the stylesheet id
		$query = "
			SELECT value 
			FROM " . cms_db_prefix() . "module_guestbook_settings 
			WHERE setting = 'template_id'
		";
		$result =& $db->Execute($query);
		if ($result)
		{
			if ($result->RecordCount() == 0)
			{
				// value not set, stylesheet wasn't inserted
				$this->InsertStylesheet();
			}
			else
			{
				$css_id = $result->Fields('value');
				# update the stylesheet
				$css_text = 
					"/*\n" . 
					$this->Lang('module_example_stylesheet_intro') . 
					"\n*/\n\n" .
					file_get_contents(
						cms_join_path($this->GetModulePath(),'css','stylesheet.css'
						)
					);
				$query = "
					UPDATE " . cms_db_prefix() . "css 
					SET css_text = '$css_text' 
					WHERE css_id = '$css_id'
				";
				$result =& $db->Execute($query);
			}
		} else {
			// db error
		}
		
		# create events
		if (method_exists($this, 'CreateEvent'))
		{
			foreach (array('EntryPosted', 'EntryDeleted', 'EntryActivated', 'EntryDeactivated') as $event)
			{
				$this->CreateEvent('Guestbook' . $event);
			}
		}
		$current_version = '1.1.3';
	case '1.1.3':
		$dict = NewDataDictionary($db);
		$flds = '
			ip_address C(11) NOTNULL
		';
		$taboptarray = array('mysql' => 'TYPE=MyISAM');
		$sqlarray = 
			$dict->AddColumnSQL(
			cms_db_prefix().'module_guestbook', 
			$flds, 
			$taboptarray
		);
		$dict->ExecuteSQLArray($sqlarray);
		
		$this->CreatePermission('Modify Guestbook Settings' , 'Modify Guestbook Settings');
		$this->SetPreference('wysiwyg', '0');
		
		$current_version = '1.1.4';
	case '1.1.4':
		if (! $form_template_is_installed)
		{
			$this->SetTemplate('guestbook-form', GetGuestbookHtmlTemplate('guestbook-form',$this->Lang('error_filenotfound_defaulttemplate')));
			$form_template_is_installed = true;
		}
		$current_version = '1.1.5';
	case '1.1.5':
		if (! $form_template_is_installed)
		{
			$this->SetTemplate('guestbook-form', GetGuestbookHtmlTemplate('guestbook-form',$this->Lang('error_filenotfound_defaulttemplate')));
			$form_template_is_installed = true;
		}
		$current_version = '1.1.6';
	case '1.1.6':
		$current_version = '1.1.7';
	case '1.1.7':
		$dict = NewDataDictionary($db);
		$fields = 'ip_address C(15) NOTNULL';
		$sqlarray = $dict->AlterColumnSQL(cms_db_prefix() . 'module_guestbook', $fields);
		$dict->ExecuteSQLArray($sqlarray);
		
		$current_version = '1.1.8';
	case '1.1.8':
		$current_version = '1.1.9';
	case '1.1.9':
		$current_version = '1.1.10';
	case '1.1.10':
		$current_version = '1.1.11';
	case '1.1.11':
		$current_version = '1.1.12';
	case '1.1.12':
		// Create the guestbook table
		$dict = NewDataDictionary($db);
		$guestbook_table = cms_db_prefix() . 'module_guestbook_guestbook';
		$flds = '
			id I KEY NOTNULL,
			name C(255) NOTNULL,
			descr C(255) NOTNULL,
			active I1 DEFAULT 0 NOTNULL
		';
		$taboptarray = array('mysql' => 'TYPE=MyISAM');
		$sqlarray = $dict->CreateTableSQL($guestbook_table, $flds, $taboptarray);
		$dict->ExecuteSQLArray($sqlarray);
				
		// Insert the default Guestbook
		$guestbook_name = $this->Lang('default_guestbook_name');
		$guestbook_descr = $this->Lang('default_guestbook_descr');
		$default_guestbook_id = $db->GenID(cms_db_prefix() . 'module_guestbook_guestbook_seq');
		$sql = "
			INSERT INTO $guestbook_table 
			SET id = $default_guestbook_id, name = '$guestbook_name', descr = '$guestbook_descr'
		";
		$db->Execute($sql);
		
		// Insert the new column "guestbook" to the guestbook(entry) table
		$dict = NewDataDictionary($db);
		$sqlarray = $dict->AddColumnSQL(cms_db_prefix()."module_guestbook", "guestbook I(11)");
		$dict->ExecuteSQLArray($sqlarray);
		
		// Import older entries in default guestbook
		$entry_table = cms_db_prefix() . 'module_guestbook';
		$sql = 'UPDATE ' . $entry_table . ' SET `guestbook` = '.$default_guestbook_id.' WHERE ' . $entry_table . '.`guestbook` is NULL;';
		$db->Execute($sql);
		
		$this->SetPreference('guestbook', $default_guestbook_id);
		$this->SetPreference('internal_smileys', '0');
		$this->SetPreference('admin_entries_per_page', '20');
		
		$current_version = '1.2';
	case '1.2':
		$current_version = '1.3.0';
	case '1.3.0':
        $this->SetPreference('dateformat', '%d-%m-%Y %H:%M');
		$current_version = '1.3.1';
	case '1.3.1':
		$query = "
			SELECT value 
			FROM " . cms_db_prefix() . "module_guestbook_settings 
			WHERE setting = 'template_id'
		";
		$result =& $db->Execute($query);
		if ($result)
		{
			if ($result->RecordCount() == 0)
			{
				// value not set, stylesheet wasn't inserted
				$this->InsertStylesheet();
			}
			else
			{
				$css_id = $result->Fields('value');
				# update the stylesheet
				$query = "
					UPDATE " . cms_db_prefix() . "css 
					SET media_type = 'handheld, screen, tv' 
					WHERE css_id = '$css_id'
				";
				$result =& $db->Execute($query);
			}
		} else {
			// db error
		}
		$current_version = '1.3.2';
	default:
	{
	}
}

#vim:ts=4 sw=4 noet
?>
