<?php

/**
 * Project:     Pagination's class: based on SmartyPaginate
 * File:        function.paginate_middle.php
 * 
 * @author Gabriel MATIAS <gabr.matias [at gmail] somethingagainstspam [and tld:] com> 
 * @author Monte Ohrt <monte at newdigitalgroup dot com>
 * @package Pagination's class
 */

function smarty_function_paginate_middle($params, &$smarty) {
    
    $_id = 'default';
    $_prefix = '[';
    $_suffix = ']';
    $_link_prefix = '';
    $_link_suffix = ''; 
    $_page_limit = null;
    $_attrs = array();
        
    foreach($params as $_key => $_val) {
        switch($_key) {
            case 'instance':
                if (!Pagination::validArray($_val)) {
                    $smarty->trigger_error('paginate_prev: bad pagination instance');
                    return;
                }
                $paginate = $_val;
                break;
            case 'prefix':
                $_prefix = $_val;
                break;
            case 'suffix':
                $_suffix = $_val;
                break;
            case 'link_prefix':
                $_link_prefix = $_val;
                break;
            case 'link_suffix':
                $_link_suffix = $_val;
                break; 
            case 'page_limit';
                $_page_limit = $_val;
                break;
            case 'format':
                break;
            case 'text':
                break;
            default:
                $_attrs[] = $_key . '="' . $_val . '"';
                break;   
        }
    }
    
    if ($paginate['total'] === false) {
        $smarty->trigger_error("paginate_middle: empty total");
        return;        
    }
                
    $_url = $paginate['url'];
    
    $_total = $paginate['total'];
    $_curr_item = $paginate['current_item'];
    $_limit = $paginate['limit'];
    
    $_item = 1;
    $_page = 1;
    $_display_pages = 0;
    $_ret = '';
    
    $_attrs = !empty($_attrs) ? ' ' . implode(' ', $_attrs) : '';
    
    if(isset($_page_limit)) {
        // find halfway point
        $_page_limit_half = floor($_page_limit / 2);
        // determine what item/page we start with
        $_item_start = $_curr_item - $_limit * $_page_limit_half;
        if( ($_view = ceil(($_total - $_item_start) / $_limit)) < $_page_limit) {
            $_item_start -= ($_limit * ( $_page_limit - $_view ));
        }
        $_item = ($_item_start >= 1) ? $_item_start : 1;
        $_page = ceil($_item / $_limit);
    }
            
    while($_item <= $_total) {
        if(isset($params['format']) && $params['format'] == 'page') {
            $_text = $_prefix . $_page . $_suffix;            
        } else {
            $_text = $_prefix . $_item . '-';
            $_text .= ($_item + $_limit - 1 <= $_total) ? $_item + $_limit - 1 : $_total;
            $_text .= $_suffix;
        }
        if($_item != $_curr_item) {
            $_this_url = $_url;
            $_this_url .= (strpos($_url, '?') === false) ? '?' : '&';
            $_this_url .= $paginate['urlvar'] . '=' . $_page;
            $_ret .= $_link_prefix . '<a href="' . $_this_url . '"' . $_attrs . '>' . $_text . '</a>' . $_link_suffix;
        } else {
            $_ret .= $_link_prefix . $_text . $_link_suffix;
        }
        $_item += $_limit;
        $_page++;
        $_display_pages++;
        if(isset($_page_limit) && $_display_pages == $_page_limit)
            break;
    }
    
    return $_ret;
    
}

?>
